# Copyright (C) 2007 Canonical Ltd
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""Helper functions for pushing and updating a remote tree."""

import subprocess

from bzrlib import (
    branch,
    bzrdir,
    errors,
    transport,
    )


def push_and_update_helper(location=None):
    """Push to a remote location, and update it as well"""
    local_branch, relpath = branch.Branch.open_containing('.')

    saved = ''
    if location is None:
        stored_loc = local_branch.get_push_location()
        if stored_loc is None:
            raise errors.BzrCommandError('no push location known or specified.')
        else:
            saved = ' saved'
        location = stored_loc
    print 'Updating%s location: %s' % (saved, location,)

    local_branch.lock_read()
    try:
        _push_and_update(local_branch, location)
    finally:
        local_branch.unlock()


def _get_target_branch(local_branch, location):
    """This will grab the target location, or create a new one."""
    try:
        target_branch = branch.Branch.open(location)
    except errors.NotBranchError:
        location = str(location)
        target_branch = local_branch.bzrdir.create_branch_and_repo(location)
    return target_branch


def _push_and_update(local_branch, location):
    target_branch = _get_target_branch(local_branch, location)
    target_branch.lock_write()
    try:
        start_revno, start_num_revs = _get_info(target_branch)
        _push_revisions(local_branch, target_branch)
        end_revno, end_num_revs = _get_info(target_branch)
    finally:
        target_branch.unlock()

    print 'Pushed %s revisions (%s mainline)' % (end_num_revs - start_num_revs,
                                                 end_revno - start_revno)

    _update_remote_tree(target_branch)


def _get_info(a_branch):
    """Figure out how long the remote ancestry is."""
    starting_revno = a_branch.revno()
    if starting_revno is 0:
        return 0, 0
    last_rev = a_branch.last_revision()
    anc = a_branch.repository.get_ancestry(last_rev)
    # anc always includes 'None'
    return starting_revno, len(anc)-1


def _push_revisions(local_branch, target_branch):
    push_func = getattr(local_branch, 'push', None)
    if push_func is not None:
        # bzr-0.15 introduces Branch.push() instead of only having
        # Branch.pull()
        push_func(target_branch)
    else:
        target_branch.pull(local_branch)


def _update_remote_tree(target_branch):
    """Update the remote working tree if it exists."""
    try:
        target_wt = target_branch.bzrdir.open_workingtree()
    except errors.NoWorkingTree:
        # TODO: we could do "cd path; bzr checkout" if we wanted to
        print 'No working tree to update at %s' % (target_branch.base,)
    except errors.NotLocalUrl:
        # we have something to update
        update_remote_branch(target_branch)
    else:
        # Push will update a local url
        return


def update_remote_branch(target_branch):
    """Run 'bzr update' in a remote branch."""
    (scheme, user, password, host,
     port, path) = transport.split_url(target_branch.base)

    if user:
        user = '%s@' % (user,)
    else:
        user = ''

    if port:
        port = ':%s' % (port,)
    else:
        port = ''

    cmd = ['ssh', user+host+port, 'bzr', 'update', path]
    print 'running "%s"' % (' '.join(cmd))

    subprocess.call(cmd)
